"""
show_laserbeacon.py defines functions showing laserbeacon device info.
"""

from Tkinter import *
import thread
import time
import math


class show_laserbeacon (Tk):
    """
    GUI class for laserbeacon devices.
    """

    def __init__(self, player, index=0, speed=10, master=None):
        """
	create a separate window using Tkinter widgets.
	create a thread for periodic sensory data update.
	"""
	# save the configuration
	self.__player = player
	self.__device_index = index
	self.__update_speed = speed
	# create a gui
	Tk.__init__(self)
	self.title('Laserbeacon : ' + str(self.__device_index))
	self.__win = Canvas(self, bg='black', width=480, height=240)
	self.__num = Label(self, text='0')
	# set layouts of widgets
	self.__win.grid(row=0, column=0, columnspan=2, sticky=N+E+S+W)
	Label(self, text='Number of Laserbeacons : ').grid(row=1, column=0, sticky=E)
	self.__num.grid(row=1, column=1, sticky=W)
	self.grid()
	# draw equidistance circles
	for idx in range(40,301,40):
	    self.__win.create_arc(240-idx, 240-idx, 240+idx, 240+idx,
	                          start=0, extent=180, outline='gray40')
	# generate rectangle items
	self.__boxes = []
	for idx in range(32):
	    self.__boxes.append(self.__win.create_rectangle(0,0, 0,0, fill='yellow'))
	self.__shown = 0
	# repository for text items
	self.__texts = []
	# start an update thread
	self.__update_flag = 1
	thread.start_new_thread(self.__update_thread, ())


    def configure(self, index=0, speed=10):
        """
	change the configuration on the fly.
	"""
	self.__device_index = index
	self.__update_speed = speed


    def close(self):
        """
	close the window, and kill it self.
	"""
	self.__update_flag = 0


    def __update_thread(self):
        """
	update sensory information.
	"""
	if self.__player.debug: print '<start a laserbeacon update thread>'
	while self.__update_flag:
	    try:
		# update canvas window
		count = self.__player.laserbeacon[self.__device_index][0]
		# delete old text items
		for item in self.__texts: self.__win.delete(item)
		# draw new text & rectangle items
		for idx in range(0, count):
		    # calculate the position
		    index = idx * 4 + 1
		    a = math.pi/180.0*(self.__player.laserbeacon[self.__device_index][index+2] + 90)
		    x = 240 + self.__player.laserbeacon[self.__device_index][index+1]*240/6000.0 *\
		               math.cos(a)
		    y = 240 - self.__player.laserbeacon[self.__device_index][index+1]*240/6000.0 *\
		              math.sin(a)
		    # show the box
		    self.__win.coords(self.__boxes[idx], x-15, y-10, x+15, y+10)
		    # draw the text
		    if self.__player.laserbeacon[self.__device_index][index] > 0:
		        self.__texts.append(self.__win.create_text(x, y,
			    text='x%02x' % self.__player.laserbeacon[self.__device_index][index]))
		# hide old boxes
		if count < self.__shown:
		    for idx in range(count, self.__shown+1):
			self.__win.coords(self.__boxes[idx], 0, 0, 0, 0)
		# update the count information
		if count != self.__shown:
		    self.__num.configure(text=count)
		    self.__shown = count
		# slow down
		time.sleep(1.0 / self.__update_speed)
	    except KeyError:
	        print '<laserbeacon:%d is not enabled yet>' % self.__device_index
		self.close()
	    except AttributeError:
		print '<invalid player object>'
	        self.close()
	    except TclError:
		self.close()
	if self.__player.debug: print '<the laserbeacon update thread finishes>'

